;****************************************************************************
;* Aufbauend auf dem 40 x 24 LCD Terminal / LCD Controller   by Benedikt   *
;* jetzt fr den ATTINY25                                                   *
;* Modifikationen by Nukeman   Mail:  nukeman69 (aeht) arcor.de             *
;*                                                                          *
;****************************************************************************
;*                                                                          *
;*  Die Software darf frei kopiert und verndert werden, solange sie nicht  *
;*    ohne meine Erlaubnis fr kommerzielle Zwecke eingesetzt wird.         *
;*                                                                          *
;****************************************************************************
;*                                                                          *
;*    Diese Software steuert ein 320x240 LCD an, und zeigt auf diesem       *
;*    durch Skalierung eines 6(8)x8 Fonts 13x5 Zeichen an.                  *
;*    Auch andere Konfigurationen lassen sich durch einfache Programm-      *
;*    nderung erreichen. ( Grsseres Display, mehr/weniger Zeilen etc.)    *
;*                                                                          *
;*    Ein Bootloader passt auch noch rein, falls die M-Option genutzt wird  *
;*    und man somit per Standard-ISP nicht mehr an den Chip rankommt.       *
;*                                                                          *
;*    UART-Protokoll:                                                       *
;*    0x00      : Cls+Home                                                  *
;*    0x01      : Home                                                      *
;*    0x02..0x09: User-Character flashen                                    *
;*                Bsp: 02 5A Row0 Row1 Row2 Row3 Row4 Row5 Row6 Row7        * 
;*                Die 5A soll verhindern, dass ein User-Zeichen aus         *
;*                Versehen umgeflasht werden.                               *
;*                Da Online programmiert wird, bitte jedem Byte ein wenig   *
;*                Zeit geben. Also z.B. nur alle 100ms ein Byte auf den Bus *
;*                geben. ( Spec sagt, eine Page braucht <4.5ms )            *
;*    0x20..0xAF: Zeichen ins Display-RAM schreiben                         *
;*                                                                          *
;****************************************************************************

/* Pin-Mapping

                  +----_----+
           M/Res -|/Res  Vcc|- +5V
             LP  -|PB3   PB2|- UART-RX
             FLM -|PB4   PB1|- Dx ( mit D3..D0 verbinden, ganz Vorsichtige entkoppeln die einzelnen Kanle per Wid.)
             GND -|Gnd   PB0|- CP   /  Bootloader-TX
                  +---------+   
                   ATTINY25

Fuses:
-------------------------------------------------------
 -U lfuse:w:0xf1:m -U hfuse:w:0xdd:m -U efuse:w:0xfe:m 
-------------------------------------------------------
= 16.5MHz PLL with max. Startup, Reset still ENABLED, SPIEN enabled, Brown-out at 2.7V, Self programming enabled

For M at Reset-Pin use:
!!!DANGER!!!!:  -U lfuse:w:0xf1:m -U hfuse:w:0x5d:m -U efuse:w:0xfe:m     !!!DANGER!!!!

*/

.include "tn25def.inc"

; Feature-Schalter
#define USER_CHAR_WRITE_incode  1  ; Kostet 112 Bytes Flash-ROM
.equ PAGE_SIZE_FLASH =         16  ; 16 Words = 32 Bytes  Page-Buffer im Tiny25

#define CLS_incode              1  ; Kostet 20 Bytes Flash
#define Reduce_Font_4_Bootldr   1  ; Letzte 2 Bytes vom Font weglassen? FBoot braucht die Adressen 0x05FE und 0x05FF selbst.
                                   ; Dadurch werden die ASCII-Zeichen 0x9E und 0x9F in ihrer jeweils letzten Pixel-Zeile verkrppelt

; Konfiguration
.equ F_CPU      =16500000          ; CPU Freq PLL-Clock, nur fuer Baudrate interessant
.equ Baudrate   =19200             ; Baudrate
.equ CSize      =8                 ; Font-Character-Hoehe 8
.equ XSize      =13                ; 13 Zeichen pro Zeile in X-Richtung darstellen
.equ YSize      =5                 ; ok4  gut5 Zeilenanzahl
.equ Y_SCALE    =6                 ; ok7  gut6 repeat each font-pixel-row this many times


; Benutzte Register
; r1:r0 werden fuers Flash-Schreiben benutzt
.def regsave     =r2
.def X_Cnt       =r3
.def YScale      =r4
.def rxready     =r5

.def XSaveL      =r6
;.def XSaveH     =r7  r7 muss unbelegt bleiben, da auf dieses Doppel-Register zugegriffen wird

#if USER_CHAR_WRITE_incode
.def zl_wr_adr   =r9
#endif

.def tempi       =r16
.def Y_Cnt       =r17
.def C_Cnt       =r18
.def temp        =r19
.def temp2       =r20
.def RXint       =r21
.def RX          =r22
.def PortCpyCH   =r23
;zustzlich: X,Y und Z  = r26..r32
; X = DDRAM-Pointer for Output
; Z = Flash Pointer auf ROM-Zeichensatz
; Y = Schreibpointer ins Display-RAM


;PortB               Eval-Board J4
.equ CLOCK      =0   ;  P14     ; Pixelclock
.equ DX         =1   ;  P15     ; data output, multiply to D3..D0
.equ UART_RX    =2   ;  P16     ; UART-Sw-Rx ( Int 0 )
.equ LP         =3   ;  P13     ; Latch Pulse (HSync)
.equ FRAME      =4   ;  P12     ; First Line Marker (VSync)
.equ M          =5   ;  ISP     ; AC Drive ( optional on Reset-Pin, ATTENTION: chip is not accessible via ICP after configuring,
                                ;            so better have a bootloader installed or a HV-Pgmer, if you want to change anything later )

.equ Baudticks1B  =(F_CPU/Baudrate)/8        ; how many ticks for 1   bit with cpu/8 divider
.equ Baudticks15B =(3*(F_CPU/Baudrate)/2)/8  ; how many ticks for 1.5 bit with cpu/8 divider

.dseg
DDRAM:    .byte    XSize*YSize  ; DDRAM


;****************************************************************************
;    Interrupt Vektoren
;****************************************************************************

.cseg
.org 0
    rjmp Reset

.org INT0addr
    rjmp UART_START_DET_IRQ

.org OC0Aaddr      ;   Bit Rest-Decoding
    rjmp UART_BIT_RCV_IRQ


;****************************************************************************
;    UART-Interrupts
;****************************************************************************
UART_START_DET_IRQ:         ; Fallende Flanke detektiert
    in   regsave, sreg
    
    clr  tempi
    out  TCNT0,tempi        ; Timer0 resetten
    out  GIMSK,tempi        ; Int0 erstmal abschalten, soll jetzt erstmal nicht mehr mitzappeln
                            ; alles in GIMSK (PCINT) wird als Kollateral-Schaden mit ausgeknipst,aufpassen,wenn da was benutzt werden soll

    ldi  tempi, Baudticks15B
    out  OCR0A, tempi       ; Timer0 auf 1,5 Bit Wartezeit laden -> in Bitmitte vom ersten Bit b0 abtasten
    ldi  tempi,(1<<OCF0A) 
    out  TIFR,tempi         ; ggf. noch anstehenden OCF0A lschen
                            ; 
    ldi  tempi, (1<<OCIE0A) ;
    out  TIMSK, tempi       ; dann Timer0-OC0A-Interrupt freigeben

    ldi  RXint,0x80         ; Receiver-Byte leeren und "letztes Bit"-Kennung einbauen
exit_det:
    out  sreg, regsave
    reti

;****************************************************************************
UART_BIT_RCV_IRQ:
    in   regsave, sreg

    ldi  tempi, Baudticks1B  ; jetzt nur noch 1-Bit Pausen machen
    out  OCR0A, tempi
    
    clc                      ; gleich ein Null-Bit als B7 reinsaugen
    ror  RXint               ; Empfangsdaten eins schieben, LSB kommt zuerst ber die Leitung
    sbic PINB, UART_RX       ; Pin low? -> nichts am RX-Byte machen
    ori  RXint,0x80          ; Pin high -> 1 einfgen

    brcc bit_rcv_exit        ; ist die 0x80 noch nicht im Carry angekommen (vom ror) ? dann raus

    rol  rxready             ; ready-flag setzen, gesetztes Carry dort reinpfeifen
    mov  RX,RXint            ; in output-buffer verschieben, dann knnen wir im int schon wieder empfangen, 
                             ; main reagiert so schnell, Buffern wre eig. nicht ntig

    in   tempi, TIMSK
    cbr  tempi, (1<<OCIE0A)
    out  TIMSK, tempi        ; Timer0-OC0A-Interrupt wieder loeschen

    ldi  tempi, (1<<INTF0)   ; clear pending Int0-Bits from last pin-changes
    out  GIFR,  tempi 

    ldi  tempi, (1<<INT0)    ; dafuer Edge high->low wieder scharf schalten
    out  GIMSK,tempi

bit_rcv_exit:
    out  sreg, regsave
    reti

;****************************************************************************
;    CPU Register Initialisierung
;****************************************************************************

Reset:
    ldi  temp, low(RAMEND)
    out  SPL, temp                 ; setup stack pointer
;    ldi    temp, high(RAMEND)
;    out    SPH, temp              ; setup stack pointer high, haben wir im T25 nicht

    ldi temp, 0x3B                 ; B5..B0= outputs except B2 = RX = Input
    out DDRB, temp

    ; Timer 0 macht den Baudratenrakt
    ldi temp, (1<<WGM01)|(0<<WGM00)
    out TCCR0A,temp                    ; Mode CTC
    ldi temp, (0<<WGM02)|2             
    out TCCR0B,temp  ;                 ; prescaler 1:8
    ; OCR0A und IRQ-Freigabe muss sowieso aus dem INT0 programmiert werden, das lassen wir hier erstmal weg

    ; Int0 auf Falling-Edge frs Startbit programmieren und bei der Gelegenheit die Pullups disablen ( dann muss RX nicht dagegen arbeiten )
    ldi temp, (1<<PUD)|(1<<ISC01)|(0<<ISC00)
    out MCUCR, temp

    ldi temp, (1<<INT0)                ; int0 enable ( sei fehlt jetzt noch )
    out GIMSK,temp

;****************************************************************************
;    Variablen Initialisierung
;****************************************************************************

    ldi YL, low(DDRAM)
    clr YH    ;ldi YH, high(DDRAM)

    ldi temp,' '-0x20 ; Fill display with font-dump
clrloop:
    st Y+, temp
    inc    temp
    ;ldi temp2, high(XSize*YSize+DDRAM)
    cpi YL, low(XSize*YSize+DDRAM)
    ;cpc YH, temp2
    brne clrloop
    
    ldi  YL,low(DDRAM) ; Schreibpointer auf Anfang setzen
    clr  rxready

    ;ldi XL, low(DDRAM) ; fngt sich sowieso nach dem ersten Frame, weglassen
    ;clr XH  ;, high(DDRAM) = always zero, we only got 128 Bytes
    
    sei  ; interrupts freigeben

;****************************************************************************
;    Hauptprogramm
;****************************************************************************

Mainloop:
    sbrs rxready,0
    rjmp LCD_Line
;****************************************************************************
; UART-Receive
;****************************************************************************
    clr   rxready
    cpi   RX,0x20          ; Display-Data? 0x20..0xFF
    brsh  StoreByte       

#if CLS_incode
    cpi   RX, 0            ; CLS / Home
    brne  cmd1
    rcall cleardisp
    rjmp  LCD_Line
cmd1:
#endif
    cpi   RX, 1            ; Home
    brne  cmd2             ; Byte wegschreiben
    ldi   YL, low(DDRAM)   ; mit 1 Schreibpointer auf Home setzen
    rjmp  LCD_Line
cmd2:
#if USER_CHAR_WRITE_incode
    cpi   RX,10            ; 2..9 = User Font Character 0..7 im Flash speichern
    brsh  Mainloop         ; Kein Kommando -> Empfangenes Zeichen verwerfen, war irgendwelcher Schrott 10..32
    rcall WriteCharFlash   ; weitere Bytes ins Flash wegschreiben
    rjmp  LCD_Line
#endif


StoreByte:
    subi RX, 32            ; 0x20er offset abziehen
    andi RX, 0x7F          ; keine Zeichen oberhalb 0x7F zulassen, sonst gibts Display-Matsch
    st   Y+, RX            ; Daten wegschreiben
    ;ldi  temp, high(XSize*YSize+DDRAM)
    cpi  YL, low(XSize*YSize+DDRAM)
    ;cpc  YH, temp
    brlo LCD_Line
    ldi  YL, low(DDRAM)
    ;ldi  YH, high(DDRAM)

;**************************************************************************************************************
LCD_Line:                   ; Bildbeginn
    ldi  temp, XSize
    mov  X_Cnt, temp
    in   PortCpyCH, PORTB       ; Aktuellen Zustand von PortB holen
    sbr  PortCpyCH, (1<<CLOCK)  ; Kopie immer mit ClockHigh konfigurieren

    movw XL, XSaveL        ; remember last start-display-position in DDRAM, we gotta do several lines for one ascii
    clr  XH                ; movw XH, XSaveH        ; XH is always zero

hloop:                     ; Loop fr 1 Zeile
    cbi  PORTB, CLOCK      ; PCLK L 
    andi ZL, 0x80          ; clear out last offset
    ld   temp, X+          ; ASCII-Offset aus RAM lesen
    or   ZL,temp           ; zu aktuellem Offset 0..7F dazuodern
; Pixel-Zeile als Block ausgeben 
    lpm  temp, Z           ; Byte aus CGROM lesen

; jetzt die 6 Bit des Characters durchgehen und 4er-Blcke setzen
    bst  temp,7            ; 1 read temp.7 to T
    bld  PortCpyCH,DX      ; 1 store T in data-line
    out  PORTB, PortCpyCH  ; 1 PCLK H + Data
    bst  temp,6            ; 1 nchstes Bit schonmal hier nach T kopieren, verlngert Clock-Puls und schadet nicht
    cbi  PORTB,CLOCK       ; 2 clock hart zurcknehmen B7 // -> 6 Zyklen pro 4er Block

    bld  PortCpyCH,DX      ; temp.6 aus T laden
    out  PORTB, PortCpyCH  ; PCLK H + Data
    bst  temp,5            ; nchstes Bit schonmal hier nach T kopieren, verlngert Clock-Puls und schadet nicht
    cbi  PORTB,CLOCK       ; clock hart zurcknehmen   B6

    bld  PortCpyCH,DX      ; temp.5 aus T laden
    out  PORTB, PortCpyCH  ; PCLK H + Data
    bst  temp,4            ; nchstes Bit schonmal hier nach T kopieren, verlngert Clock-Puls und schadet nicht
    cbi  PORTB,CLOCK       ; clock hart zurcknehmen

    bld  PortCpyCH,DX      ; temp.4 aus T laden
    out  PORTB, PortCpyCH  ; PCLK H + Data
    bst  temp,3            ; nchstes Bit schonmal hier nach T kopieren, verlngert Clock-Puls und schadet nicht
    cbi  PORTB,CLOCK       ; clock hart zurcknehmen

    bld  PortCpyCH,DX      ; temp.3 aus T laden
    out  PORTB, PortCpyCH  ; PCLK H + Data
    bst  temp,2            ; nchstes Bit schonmal hier nach T kopieren, verlngert Clock-Puls und schadet nicht
    cbi  PORTB,CLOCK       ; clock hart zurcknehmen

    bld  PortCpyCH,DX      ; temp.2 aus T laden
    out  PORTB, PortCpyCH  ; PCLK H + Data
    cbr  PortCpyCH,(1<<DX) ; Data schonmal blanken, falls leerpulse folgen
    cbi  PORTB,CLOCK       ; clock hart zurcknehmen

    dec  X_Cnt        ;
    brne hloop        ;
                      
    ; eine extra-clock raushauen wegen auflsung

    out  PORTB, PortCpyCH  ; PCLK H
    nop
    cbi  PORTB,CLOCK       ; clock hart zurcknehmen

hloopend:
    sbi  PORTB, LP         ; Zeilenpuls geben
    inc  YScale            ; Zeilenskalierer hochzhlen
    nop                    ; LP nicht zu kurz
    cbi  PORTB, LP         ; Zeilenpulse schon wieder zuende
    cbi  PORTB, FRAME      ; frame-puls zurcknehmen, falls er noch gesetzt war

    brne no_dec_c_cnt      ; y-scaler noch nicht fertig? -> dann ohne c-cnt runterzaehlen weitermachen

    ldi  temp, 256-Y_SCALE ; sonst
    mov  YScale,temp       ; Y-Scaler fr naechste Zeile neu starten
    dec  C_Cnt             ; Zaehler fr Zeichenzeilenzahl erniedrigen
    breq skipcloop         ; alle Pixelzeilen von akt. DDRAM-Textzeile fertig ausgegeben ?  ->  nchste DDRAM-Zeile

    ldi  temp, 0x80        ; Char-Adresse += 0x0080
    clr  temp2
    add  ZL,temp           ; mit CGROM-Pointer auf naechste Pixelzeile zeigen
    adc  ZH,temp2
no_dec_c_cnt:
    sbiw XL, XSize         ; ASCII-Pointer wieder an Anfang der aktuellen Textzeile setzen
    rjmp exit_line         ; und raus

skipcloop:                 ; Alles fr neue Textzeile einrichten
    ldi  C_Cnt, CSize      ; Zeichenhhen Counter frisch laden
    inc  Y_Cnt             ; Zeilencounter erhhen
    ldi  ZH, high(2*font)  ; CGROM Offset, erste Pixelzeile
    clr  ZL
    cpi  Y_Cnt, YSize      ; Bildende ?
    brne exit_line
    sbi  PINB, M           ; use Port-Toggle function to toggle M

    sbi  PORTB, FRAME      ; Bildneustart
    clr  Y_Cnt
    ldi  XL, low(DDRAM)    ; ascii-ram auf start setzen
    ;ldi  XH, high(DDRAM)  ; always zero
exit_line:
    movw XSaveL, XL        ; DDRAM-start-pointer in XL speichern, wird naechstes Mal wieder vorgeholt
;**************************************************************************************************************
    rjmp Mainloop


;**************************************************************************************************************
; Unterfunktionen
;**************************************************************************************************************

#if CLS_incode
cleardisp:
    ldi temp, ' '-0x20   ; display lschen
    ldi YL, low(DDRAM)
    ;ldi YH, high(DDRAM)
clrloopw:
    st  Y+, temp
    ;ldi temp2, high(XSize*YSize+DDRAM)
    cpi YL, low(XSize*YSize+DDRAM)
    ;cpc YH, temp2
    brne clrloopw
    ldi YL,  low(DDRAM)
    ;ldi YH, high(DDRAM)
    ret
#endif

;**************************************************************************************************************

#if USER_CHAR_WRITE_incode
WriteCharFlash:
    ; RX hat die Zeichenadresse 2..9, wo das User-Zeichen hin soll
    ; entsprechend dem ASCII-Code 0x85..0x8D, um 0x20 erniedrigt = 0x65..0x6D
    ; da stehen sowieso nur bekloppte Zeichen drin, die kaum jemand nutzt.
    push ZH                ; das Registerpaar brauchen wir gerade mal eben
    push ZL
#define temp3  PortCpyCH   ; knnen wir hier konfliktfrei ohne zu sichern benutzen

    mov   temp, RX         ; Character-Index kurz sichern

checkmagic:
    sbrs  rxready,0
    rjmp  checkmagic      ; auf nchstes Byte vom UART warten
    clr   rxready         ; Empfangsflag zuruecknehmen

    cpi   RX,0x5A         ; magic byte auch OK?  Soll vor versehentlichen Schreibern schtzen
    brne  WrFlashExit     ; falsch? -> Raus hier
   
    subi temp, -99        ; temp - (-99) = temp + 0x63  2->0x65(0x85)  9->0x6D(0x8D)

    ldi  ZH,high(2*font)  ; Font-Basis-Adresse laden, muss auf 0x0X00 liegen
    mov  ZL,temp          ; auf erste Pixel-Zeile des zu aendernden Zeichens gehen

cgloop:
    sbrs  rxready,0
    rjmp  cgloop          ; auf nchstes Byte vom UART warten
    clr   rxready

/* Adressen:
                    Zeile0  Zeile1  Zeile2  Zeile3  Zeile4  Zeile5  Zeile6  Zeile7
   Zeichen 0x65     0x0265  0x02E5  0x0365  0x03E5  0x0465  0x04E5  0x0565  0x05E5
*/
    lpm  temp2,Z          ; Byte vom Flash holen, vielleicht passt es ja schon und
    cp   RX,temp2         ; Flashen kann ausgelassen werden
    breq skip_page_write

    ; passt nicht und muss geflasht werden
    ; 1.Fill temporary page-buffer with current content from flash
    ldi temp2, PAGE_SIZE_FLASH

    push ZH               ; 
    push ZL               ; exakten Z des Adress-Zhlers merken
    mov  zl_wr_adr, ZL    ; Schreib-Adresse merken
    andi ZL,0xE0          ; ZL auf Page-Anfang setzen
fill_temp_page:
    lpm  r0,Z             ; Byte aus Flash holen
    cp   ZL, zl_wr_adr    ; muss hier das empfangene Byte vom UART hin?
    brne read_low_fm_flash
    mov  r0,RX            ; durch empfangenes Byte ersetzen
read_low_fm_flash:
    lpm  r1,Z+            ; dient nur als Pointer-Increment
    lpm  r1,Z             ; Byte aus Flash lesen
    cp   ZL, zl_wr_adr    ; muss hier das empfangene Byte hin?
    brne store_2_temp
    mov  r1,RX            ; durch empfangenes Byte ersetzen

store_2_temp:
    ldi  temp3,(1<<SPMEN) ; Save to temporary buffer
    out  SPMCSR,temp3     ; 
    spm                   ; save r1:r0 to temporary page
    lpm  r0,Z+            ; noch fehlenden Increment auf Z nachholen
    dec  temp2            ; temp-page full?
    brne fill_temp_page

    pop  ZL
    pop  ZH               ; exakten Flash-Pointer wieder herstellen

    cli                   ; mal kurz die Interrupts weglassen

    ; So, temp ist jetzt richtig gefllt, Flash-Page lschen
    ldi  temp3,(1<<PGERS)|(1<<SPMEN) ; Page Erase
    out  SPMCSR,temp3     ; 
    spm                   ; kill em all
wait_erase:
    in   temp3,SPMCSR     ; braucht das? Spec sagt, die CPU wird sowieso angehalten
    sbrc temp3,PGERS
    rjmp wait_erase


    ; Page Write
    ldi  temp3,(1<<PGWRT)|(1<<SPMEN) ; Page Write
    out  SPMCSR,temp3     ; 
    spm                   ; mit aktualisierten Daten berschreiben
wait_write:
    in   temp3,SPMCSR     ; braucht das? Spec sagt, die CPU wird sowieso angehalten
    sbrc temp3,PGWRT
    rjmp wait_erase

    sei                   ; Interrupts wieder freigeben

skip_page_write:
    subi ZL,-128          ; auf nchsten Zeilenblock schalten ( += 0x80 )
    brcs cgloop           ; weil heute Gegenteil-Tag ist, reagiert Carry invers
    inc  ZH               ; an ZH feststellen, ob wir fertig sind font geht bis 0x05FF -> 0x06 ist zuweit
    cpi  ZH,high(2*font+1024)
    brne cgloop
    
WrFlashExit:
    pop  ZL               ; Font-Pointer fuer die Zeilenroutine wieder restaurieren
    pop  ZH
    ret
#endif


;**************************************************************************************************************



;*** CG ROM Daten *** (6x8 Zeichensatz)

/* 
  Muss so abgelegt werden:

  Char0Line0  Char1Line0  Char2Line0 .. Char127Line0
  Char0Line1  Char1Line1  Char2Line1 .. Char127Line1
   
  Char0Line7  Char1Line7  Char2Line7 .. Char127Line7

  Pixel-Zeile:  Adr-Offset:
          0       0x000
          1       0x080
          2       0x100
          3       0x180
          4       0x200
          5       0x280
          6       0x300
          7       0x380
*/

.org 512/2  ; Bei 512 Bytes ( 0x0200..0x05FF )
font: // 128*8 = 1k  ASCII: 0x20..0x9F, ist das meiste wichtige drin, 85..8D knnen als User-Zeichen geschrieben werden
.db 0x00,0x10,0x6c,0x00,0x20,0x64,0x20,0x30,0x10,0x20,0x00,0x00,0x00,0x00,0x00,0x00
.db 0x38,0x10,0x38,0x38,0x08,0x7c,0x18,0x7c,0x38,0x38,0x00,0x00,0x08,0x00,0x20,0x38
.db 0x38,0x38,0x78,0x38,0x78,0x7c,0x7c,0x38,0x44,0x38,0x04,0x44,0x40,0x44,0x44,0x38
.db 0x78,0x38,0x78,0x38,0x7c,0x44,0x44,0x44,0x44,0x44,0x78,0x38,0x00,0x38,0x10,0x00
.db 0x30,0x00,0x40,0x00,0x04,0x00,0x18,0x00,0x40,0x10,0x08,0x40,0x10,0x00,0x00,0x00
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x10,0x30,0x28,0x10
.db 0x38,0x48,0x0c,0x38,0x28,0x30,0x38,0x00,0x38,0x28,0x30,0x28,0x38,0x20,0x28,0x38
.db 0x0c,0x00,0x3c,0x38,0x28,0x60,0x38,0x60,0x28,0x48,0x28,0x00,0x18,0x3c,0x00,0x08


.db 0x00,0x38,0x6c,0x28,0x38,0x64,0x50,0x30,0x20,0x10,0x28,0x10,0x00,0x00,0x00,0x04
.db 0x44,0x30,0x44,0x44,0x18,0x40,0x20,0x04,0x44,0x44,0x00,0x00,0x10,0x00,0x10,0x44
.db 0x44,0x44,0x44,0x44,0x44,0x40,0x40,0x44,0x44,0x10,0x04,0x48,0x40,0x6c,0x64,0x44
.db 0x44,0x44,0x44,0x44,0x10,0x44,0x44,0x44,0x44,0x44,0x08,0x20,0x40,0x08,0x28,0x00
.db 0x30,0x00,0x40,0x00,0x04,0x00,0x20,0x00,0x40,0x00,0x00,0x40,0x10,0x00,0x00,0x00
.db 0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x10,0x08,0x50,0x38
.db 0x44,0x00,0x00,0x00,0x00,0x00,0x28,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x28
.db 0x00,0x00,0x50,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x24,0x4c,0x44,0x14


.db 0x00,0x38,0x48,0x7c,0x40,0x08,0x50,0x20,0x20,0x10,0x38,0x10,0x00,0x00,0x00,0x08
.db 0x4c,0x10,0x04,0x04,0x28,0x40,0x40,0x08,0x44,0x44,0x30,0x30,0x20,0x7c,0x08,0x04
.db 0x5c,0x44,0x44,0x40,0x44,0x40,0x40,0x40,0x44,0x10,0x04,0x50,0x40,0x54,0x54,0x44
.db 0x44,0x44,0x44,0x40,0x10,0x44,0x44,0x54,0x28,0x44,0x10,0x20,0x20,0x08,0x44,0x00
.db 0x10,0x38,0x78,0x38,0x3c,0x38,0x20,0x3c,0x70,0x10,0x18,0x48,0x10,0x68,0x70,0x38
.db 0x78,0x3c,0x58,0x38,0x78,0x48,0x44,0x44,0x48,0x48,0x78,0x20,0x10,0x08,0x00,0x6c
.db 0x40,0x48,0x38,0x38,0x38,0x38,0x38,0x44,0x38,0x38,0x38,0x10,0x10,0x10,0x10,0x38
.db 0x7c,0x78,0x50,0x30,0x30,0x30,0x48,0x48,0x48,0x48,0x48,0x04,0x20,0x54,0x28,0x10


.db 0x00,0x10,0x00,0x28,0x30,0x10,0x20,0x00,0x20,0x10,0x7c,0x7c,0x00,0x7c,0x00,0x10
.db 0x54,0x10,0x18,0x38,0x48,0x78,0x78,0x10,0x38,0x3c,0x30,0x30,0x40,0x00,0x04,0x18
.db 0x54,0x44,0x78,0x40,0x44,0x78,0x78,0x5c,0x7c,0x10,0x04,0x60,0x40,0x44,0x4c,0x44
.db 0x78,0x44,0x78,0x38,0x10,0x44,0x44,0x54,0x10,0x28,0x20,0x20,0x10,0x08,0x00,0x00
.db 0x00,0x04,0x44,0x44,0x44,0x44,0x78,0x44,0x48,0x10,0x08,0x50,0x10,0x54,0x48,0x44
.db 0x44,0x44,0x24,0x40,0x20,0x48,0x44,0x44,0x48,0x48,0x08,0x60,0x00,0x0c,0x00,0x44
.db 0x40,0x48,0x44,0x04,0x04,0x04,0x04,0x40,0x44,0x44,0x44,0x10,0x10,0x10,0x28,0x6c
.db 0x40,0x14,0x7c,0x48,0x48,0x48,0x48,0x48,0x48,0x48,0x48,0x38,0x78,0x54,0x10,0x38


.db 0x00,0x10,0x00,0x28,0x08,0x20,0x54,0x00,0x20,0x10,0x38,0x10,0x00,0x00,0x00,0x20
.db 0x64,0x10,0x20,0x04,0x7c,0x04,0x44,0x20,0x44,0x04,0x00,0x00,0x20,0x00,0x08,0x10
.db 0x5c,0x7c,0x44,0x40,0x44,0x40,0x40,0x44,0x44,0x10,0x44,0x50,0x40,0x44,0x44,0x44
.db 0x40,0x54,0x48,0x04,0x10,0x44,0x44,0x54,0x28,0x10,0x40,0x20,0x08,0x08,0x00,0x00
.db 0x00,0x3c,0x44,0x40,0x44,0x78,0x20,0x44,0x48,0x10,0x08,0x60,0x10,0x54,0x48,0x44
.db 0x44,0x44,0x20,0x38,0x20,0x48,0x44,0x54,0x30,0x48,0x30,0x20,0x10,0x08,0x00,0x44
.db 0x44,0x48,0x78,0x3c,0x3c,0x3c,0x3c,0x44,0x78,0x78,0x78,0x10,0x10,0x10,0x44,0x44
.db 0x78,0x7c,0x50,0x48,0x48,0x48,0x48,0x48,0x48,0x48,0x48,0x58,0x20,0x54,0x28,0x10


.db 0x00,0x00,0x00,0x7c,0x70,0x4c,0x48,0x00,0x20,0x10,0x28,0x10,0x30,0x00,0x30,0x40
.db 0x44,0x10,0x40,0x44,0x08,0x44,0x44,0x20,0x44,0x08,0x30,0x30,0x10,0x7c,0x10,0x00
.db 0x40,0x44,0x44,0x44,0x44,0x40,0x40,0x44,0x44,0x10,0x44,0x48,0x40,0x44,0x44,0x44
.db 0x40,0x48,0x44,0x44,0x10,0x44,0x28,0x54,0x44,0x10,0x40,0x20,0x04,0x08,0x00,0x00
.db 0x00,0x44,0x44,0x44,0x44,0x40,0x20,0x3c,0x48,0x10,0x08,0x50,0x10,0x44,0x48,0x44
.db 0x44,0x44,0x20,0x04,0x28,0x58,0x28,0x7c,0x48,0x38,0x40,0x20,0x10,0x08,0x00,0x7c
.db 0x38,0x58,0x40,0x44,0x44,0x44,0x44,0x38,0x40,0x40,0x40,0x10,0x10,0x10,0x7c,0x7c
.db 0x40,0x50,0x50,0x48,0x48,0x48,0x58,0x58,0x38,0x48,0x48,0x68,0x24,0x64,0x44,0x10


.db 0x00,0x10,0x00,0x28,0x10,0x4c,0x34,0x00,0x10,0x20,0x00,0x00,0x30,0x00,0x30,0x00
.db 0x38,0x38,0x7c,0x38,0x08,0x38,0x38,0x20,0x38,0x30,0x30,0x30,0x08,0x00,0x20,0x10
.db 0x38,0x44,0x78,0x38,0x78,0x7c,0x40,0x3c,0x44,0x38,0x38,0x44,0x7c,0x44,0x44,0x38
.db 0x40,0x34,0x44,0x38,0x10,0x38,0x10,0x28,0x44,0x10,0x78,0x38,0x00,0x38,0x00,0x00
.db 0x00,0x3c,0x78,0x38,0x3c,0x38,0x20,0x04,0x48,0x18,0x48,0x48,0x18,0x44,0x48,0x38
.db 0x78,0x3c,0x70,0x38,0x10,0x28,0x10,0x28,0x48,0x10,0x78,0x18,0x10,0x30,0x00,0x00
.db 0x10,0x28,0x38,0x3c,0x3c,0x3c,0x3c,0x10,0x38,0x38,0x38,0x18,0x18,0x18,0x44,0x44
.db 0x7c,0x3c,0x5c,0x30,0x30,0x30,0x28,0x28,0x10,0x30,0x30,0x70,0x5c,0x78,0x00,0x50


.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x00
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfc
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00
.db 0x40,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00,0x00,0x00
.db 0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
.db 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x80,0x00,0x00

#if( Reduce_Font_4_Bootldr == 0 )
.db 0x00,0x20
#endif

